.\" Copyright 1999 Sun Microsystems, Inc.  All rights reserved.
.\"
.\" Permission is hereby granted, free of charge, to any person obtaining a
.\" copy of this software and associated documentation files (the "Software"),
.\" to deal in the Software without restriction, including without limitation
.\" the rights to use, copy, modify, merge, publish, distribute, sublicense,
.\" and/or sell copies of the Software, and to permit persons to whom the
.\" Software is furnished to do so, subject to the following conditions:
.\"
.\" The above copyright notice and this permission notice (including the next
.\" paragraph) shall be included in all copies or substantial portions of the
.\" Software.
.\"
.\" THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
.\" IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
.\" FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
.\" THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
.\" LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
.\" FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
.\" DEALINGS IN THE SOFTWARE.
.\"
.TH XkbAddGeomColor __libmansuffix__ __xorgversion__ "XKB FUNCTIONS"
.SH NAME
XkbAddGeomColor \-  Add one color name to an existing keyboard geometry 
description
.SH SYNOPSIS
.HP
.B XkbColorPtr XkbAddGeomColor
.BI "(\^XkbGeometryPtr " "geom" "\^,"
.BI "char *" "spec" "\^,"
.BI "unsigned int " "pixel" "\^);"
.if n .ti +5n
.if t .ti +.5i
.SH ARGUMENTS
.TP
.I \- geom
geometry to be updated
.TP
.I \- spec
color to be added
.TP
.I \- pixel
color to be added
.SH DESCRIPTION
.LP
Xkb provides functions to add a single new element to the top-level keyboard 
geometry. In each case the 
.I num_ * 
fields of the corresponding structure is incremented by 1. These functions do 
not change 
.I sz_* 
unless there is no more room in the array. Some of these functions fill in the 
values of the element's structure from the arguments. For other functions, you 
must explicitly write code to fill the structure's elements.

The top-level geometry description includes a list of 
.I geometry properties. 
A geometry property associates an arbitrary string with an equally arbitrary 
name. Programs that display images of keyboards can use geometry properties as 
hints, but they are not interpreted by Xkb. No other geometry structures refer 
to geometry properties.

.I XkbAddGeomColor 
adds the specified color 
.I name 
and 
.I pixel 
to the specified geometry 
.I geom. 
The top-level geometry description includes a list of up to MaxColors (32) 
.I color names. 
A color 
.I name 
is a string whose interpretation is not specified by Xkb and neither is the
.I pixel 
value's interpretation. All other geometry data structures refer to colors using 
their indices in this global list or pointers to colors in this list.
.I XkbAddGeomColor 
returns NULL if any of the parameters is empty or if it was not able to allocate 
space for the color. To allocate space for an arbitrary number of colors to a 
geometry, use the 
.I XkbAllocGeomColors 
function.
.SH STRUCTURES
.LP
.nf

    typedef struct _XkbColor {
        unsigned int    pixel;      /\(** color */
        char *          spec;       /\(** color name */
    } XkbColorRec,*XkbColorPtr;
    
.fi
.SH "SEE ALSO"
.BR XkbAllocGeomColors (__libmansuffix__)

