'\" t
.\" Copyright 1999 Sun Microsystems, Inc.  All rights reserved.
.\"
.\" Permission is hereby granted, free of charge, to any person obtaining a
.\" copy of this software and associated documentation files (the "Software"),
.\" to deal in the Software without restriction, including without limitation
.\" the rights to use, copy, modify, merge, publish, distribute, sublicense,
.\" and/or sell copies of the Software, and to permit persons to whom the
.\" Software is furnished to do so, subject to the following conditions:
.\"
.\" The above copyright notice and this permission notice (including the next
.\" paragraph) shall be included in all copies or substantial portions of the
.\" Software.
.\"
.\" THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
.\" IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
.\" FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
.\" THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
.\" LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
.\" FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
.\" DEALINGS IN THE SOFTWARE.
.\"
.TH XkbAllocDeviceLedInfo __libmansuffix__ __xorgversion__ "XKB FUNCTIONS"
.SH NAME
XkbAllocDeviceLedInfo \- Obtain an XkbDeviceLedInfoRec structure
.SH SYNOPSIS
.HP
.B Status XkbAllocDeviceLedInfo
.BI "(\^XkbDeviceInfoPtr " "device_info" "\^,"
.BI "int " "num_needed" "\^);"
.if n .ti +5n
.if t .ti +.5i
.SH ARGUMENTS
.TP
.I \- device_info
structure in which to allocate LED space
.TP
.I \- num_needed
number of indicators to allocate space for
.SH DESCRIPTION
.LP
.I XkbAllocDeviceLedInfo 
allocates space for an XkbDeviceLedInfoRec and places it in
.I device_info. 
If 
.I num_needed 
is nonzero, 
.I num_needed 
XkbIndicatorMapRec structures are also allocated and linked into the XkbDeviceLedInfoRec structure. If you request XkbIndicatorMapRec structures be allocated using this request, you must initialize them explicitly. All other fields are initialized to zero.

To initialize an XkbDeviceLedInfoRec structure, use 
.I XkbAddDeviceLedInfo.
.SH STRUCTURES
.LP
Information about X Input Extension devices is transferred between a client program and the Xkb 
extension in an XkbDeviceInfoRec structure:
.nf

    typedef struct {
        char *               name;          /\(** name for device */
        Atom                 type;          /\(** name for class of devices */
        unsigned short       device_spec;   /\(** device of interest */
        Bool                 has_own_state; /\(** True=>this device has its own state */
        unsigned short       supported;     /\(** bits indicating supported capabilities */
        unsigned short       unsupported;   /\(** bits indicating unsupported capabilities */
        unsigned short       num_btns;      /\(** number of entries in btn_acts */
        XkbAction *          btn_acts;      /\(** button actions */
        unsigned short       sz_leds;       /\(** total number of entries in LEDs vector */
        unsigned short       num_leds;      /\(** number of valid entries in LEDs vector */
        unsigned short       dflt_kbd_fb;   /\(** input extension ID of default (core kbd) indicator */
        unsigned short       dflt_led_fb;   /\(** input extension ID of default indicator feedback */
        XkbDeviceLedInfoPtr  leds;          /\(** LED descriptions */
    } XkbDeviceInfoRec, *XkbDeviceInfoPtr;
    

    typedef struct {
        unsigned short      led_class;        /\(** class for this LED device*/
        unsigned short      led_id;           /\(** ID for this LED device */
        unsigned int        phys_indicators;  /\(** bits for which LEDs physically present */
        unsigned int        maps_present;     /\(** bits for which LEDs have maps in maps */
        unsigned int        names_present;    /\(** bits for which LEDs are in names */
        unsigned int        state;            /\(** 1 bit => corresponding LED is on */
        Atom                names[XkbNumIndicators];   /\(** names for LEDs */
        XkbIndicatorMapRec  maps;             /\(** indicator maps for each LED */
    } XkbDeviceLedInfoRec, *XkbDeviceLedInfoPtr;

.fi    
Each indicator has its own set of attributes that specify whether clients can explicitly set its state and whether it tracks the keyboard state. The attributes of each indicator are held in the maps array, which is an array of XkbIndicatorRec structures:
.nf

    typedef struct {
        unsigned char   flags;            /\(** how the indicator can be changed */
        unsigned char   which_groups;     /\(** match criteria for groups */
        unsigned char   groups;           /\(** which keyboard groups the indicator watches */
        unsigned char   which_mods;       /\(** match criteria for modifiers */
        XkbModsRec      mods;             /\(** which modifiers the indicator watches */
        unsigned int    ctrls;            /\(** which controls the indicator watches */
    } XkbIndicatorMapRec, *XkbIndicatorMapPtr;
    
.fi    
This indicator map specifies for each indicator:

.IP \(bu 5
The conditions under which the keyboard modifier state affects the indicator
.IP \(bu 5
The conditions under which the keyboard group state affects the indicator
.IP \(bu 5
The conditions under which the state of the boolean controls affects the indicator
.IP \(bu 5
The effect (if any) of attempts to explicitly change the state of the indicator using the functions XkbSetControls or XChangeKeyboardControl    
.SH "SEE ALSO"
.BR XkbAddDeviceLedInfo (__libmansuffix__),
.BR XkbSetControls (__libmansuffix__)
